//draws circular and linear histograms in PostScript
//Wed Jan 26 08:25:30 MET 2005 Peter A. Cejchan <cej@gli.cas.cz | tapique@tiscali.cz | pac@plan9.cz>

#include <u.h>
#include <libc.h>
#include <bio.h>
#include "plot.h"

#define HALFWIDTH 8192	/* output scaled to fit in -HALFWIDTH,HALFWIDTH */
#define MINVALUE  -1e100
#define MAXLENGTH 65536
#ifndef PI
#define PI	3.1415926535897932384626433832795028841971693993751
#endif

#define TWOPI (2*PI)

static int n;
static double f[360];
static double fmax;
static int left = -HALFWIDTH;
static int right = HALFWIDTH;
static int bottom = -HALFWIDTH;
static int top = HALFWIDTH;

Biobuf	bout;
long	flag;
enum
{
	Fcirc	= 1<<0,
	Flin = 1<<1,
	Fpoly = 1<<2,
	Fbar	= 1<<3,
};

int readdata(int f);
double findmax(void);
void drw(void);
void drw2(void);
void drw3(void);
void drw4(void);

void
main(int argc, char *argv[])
{
	int f;
	flag = 0;
	Binit(&bout, 1, OWRITE);
	ARGBEGIN{
	default:
		fprint(2, "rosie: unknown option %c\n", ARGC());
		exits("usage");
	case 'c':
		flag |= Fcirc;
		break;
	case 'l':
		flag |= Flin;
		break;
	case 'p':
		flag |= Fpoly;
		break;
	case 'b':
		flag |= Fbar;
		break;
	}ARGEND
	if((flag&(Fcirc|Flin|Fpoly|Fbar)) == 0)	flag = Fcirc  | Fpoly;

		if((flag & Fcirc)&&(flag & Flin))
		{
			fprint(2, "rosie: plot could not be both circular and linear\n");
			exits("usage");
		}
		if((flag & Fpoly)&&(flag & Fbar))
		{
			fprint(2, "rosie: plot could not be both polygon and columnar\n");
			exits("usage");
		}

	if(argc < 1) 
		readdata(0);
	else
	{
		f = open(argv[0], 0);
		if(f < 0) {
			fprint(2, "cannot open %s\n", argv[0]);
		}
		readdata(f);
		close(f);
	}

	fmax=findmax();
	openpl();
	erase();

		if((flag & Fcirc)&&(flag & Fbar)) drw();
		if((flag & Fcirc)&&(flag & Fpoly)) drw2();
		if((flag & Flin)&&(flag & Fbar)) drw3();
		if((flag & Flin)&&(flag & Fpoly)) drw4();


	closepl();

}


int 
readdata(int file)
// read whitespace-delimited vector f[i] (external)
{
	Biobuf bin;
	char *buf, *av[MAXLENGTH];
	int  j;

	Binit(&bin, file, OREAD);
	if((buf = Brdstr(&bin, '\n', 1)) != nil){
		n=tokenize(buf, av, MAXLENGTH);
		for(j=0; j<n; j++){
			f[j]=atof(av[j]);
		}
		free(buf);
	}
	return 1;
}

double
findmax()
{
int i;
double max;
	max=MINVALUE;
	for(i=0; i<n; i++)
		if(f[i]>max) max=f[i];
	return max;
}


void
drw()
{
	int i, ax, ay, bx, by, cx, cy;
	double alpha, omega, scaling;
	cx=0; cy=0;

	alpha=TWOPI/n;
	omega=-alpha/2.0;
	scaling=HALFWIDTH/fmax;
	pen(SOLID);
	range(left,bottom,right,top);

	for(i=0; i<n; i++) 
	{
		move(cx, cy);
		ax=(int)f[i]*sin(omega)*scaling;
		ay=(int)f[i]*cos(omega)*scaling;
		vec(ax, ay);
		omega+=alpha;
		bx=(int)f[i]*sin(omega)*scaling;
		by=(int)f[i]*cos(omega)*scaling;
		vec(bx, by);
		vec(cx, cy);
	}
	print("closepath\ns\nfill\n");
}

void
drw2()
{
	int i, ax, ay;
	double alpha, omega, scaling;

	alpha=TWOPI/n;
	omega=0.0;
	scaling=HALFWIDTH/fmax;
	pen(SOLID);
	range(left,bottom,right,top);

		ax=(int)f[0]*sin(omega)*scaling;
		ay=(int)f[0]*cos(omega)*scaling;
		move(ax, ay);

	for(i=0; i<n; i++) 
	{
		ax=(int)f[i]*sin(omega)*scaling;
		ay=(int)f[i]*cos(omega)*scaling;
		vec(ax, ay);
		omega+=alpha;
	}
		ax=(int)f[0]*sin(omega)*scaling;
		ay=(int)f[0]*cos(omega)*scaling;
		vec(ax, ay);

	print("closepath\ns\nfill\n");
}

void
drw3()
{
	int i, ax, ay, bx, by, cx, cy, dx, dy;
	double alpha, omega, scaling;

	alpha=2.0*HALFWIDTH/n;
	omega=0.0;
	scaling=HALFWIDTH/fmax;
	pen(SOLID);
	range(left,bottom,right,top);


	for(i=0; i<n; i++) 
	{
		ax=(int)left+omega;
		ay=(int)bottom;
		bx=ax;
		by=(int)ay+f[i]*scaling;
		cx=(int)ax+alpha;
		cy=by;
		dx=cx;
		dy=ay;
		move(ax, ay);
		vec(bx, by);
		vec(cx, cy);
		vec(dx, dy);
		vec(ax, ay);
		omega+=alpha;
	}
	print("closepath\ns\nfill\n");
}

void
drw4()
{
			fprint(2, "rosie: this type of plot is not implemented yet\n");
}